#include "ssd192x.h"
#include "utils.h"
#include "camera.h"

extern UINT32 SET_CAM_WIDTH;

/*********************************************************************************************************************************************
* void ssd192x_cam_init(INT32 crop_x, INT32 crop_y, INT32 crop_width, INT32 crop_height, INT32 vhratio, INT32 vvratio, INT32 jhratio, INT32 jvratio)      *                                              									                                             *
*                                                                                                                                                                                                                                                                                        *
* Description:                                                                                                                                                                                                                                                                  *
*                                                                                                                                                                                                                                                                                        *
*                                                                                                                                                                                                                                                                                        *
*                                                                                                                                                                                                                                                                                        *
**********************************************************************************************************************************************/

void ssd192x_cam_init(INT32 crop_x, INT32 crop_y, INT32 crop_width, INT32 crop_height, INT32 vhratio, INT32 vvratio, INT32 jhratio, INT32 jvratio)
{
	INT32 s_vhratio, s_vvratio;
	// for displaying value of 2^vxratio

	// 170 & 171 is preview ratio, which shall be 1 : 2^(reg 170/1 )
	// 172 & 173 is the line buffer size

	SSD_REGWB(REG_DV_VHDEC_RATIO,vhratio); 	//0x170
	SSD_REGWB(REG_DV_VVDEC_RATIO,vvratio); 	//0x171
	SSD_REGWB(REG_DV_JHDEC_RATIO,jhratio);	//0x172
	SSD_REGWB(REG_DV_JVDEC_RATIO,jvratio);	//0x173

	if ( vhratio == 0)
		s_vhratio = 1;
	else
		s_vhratio = vhratio*2;

	if ( vvratio == 0)
		s_vvratio = 1;
	else
		s_vvratio = vvratio*2;

	myprintf("Ratio of input to preview ( Horizontal )= 1:%d\r\n",s_vhratio);
	myprintf("Ratio of input to preview ( Vertical )  = 1:%d\r\n",s_vvratio);
	//myprintf("Line Buffer size ratio     = %d\r\n",jhratio);
	//myprintf("Vertical buffer size ratio = %d\r\n",jvratio);

	//From where we want to display from the original capture screen
	SSD_REGWW(REG_DV_HCROP_STR_0,crop_x); //0x184
	SSD_REGWW(REG_DV_VCROP_STR_0,crop_y); //0x188

	myprintf("Crop X start location from DV input = %d\r\n",crop_x);
	myprintf("Crop Y start location from DV input = %d\r\n",crop_y);

	// Set the width and height we need to display on panel
	SSD_REGWW(REG_DV_HCROP_SIZE_0,crop_width);  //0x18C
	SSD_REGWW(REG_DV_VCROP_SIZE_0,crop_height); //0x190

	myprintf("Horizontal size of DV input = %d\r\n",SSD_REGRW(REG_DV_HCROP_SIZE_0));
	myprintf("Vertical size of DV input   = %d\r\n\n",SSD_REGRW(REG_DV_VCROP_SIZE_0));

	// set the camera using the I2C
	cam_hw_init(); // code in ov7670.c

	return;
}

/************************************************************************************************************
* void ssd192x_cam_preview(UINT32 addr, INT32 on)                 				                                                                   *
*                                                                                                                                                                                                                      *
* Description:                                                                                                                                                                                                *
*                                                                                                                                                                                                                      *
*                                                                                                                                                                                                                      *
*                                                                                                                                                                                                                      *
*************************************************************************************************************/

void ssd192x_cam_preview(UINT32 addr, INT32 on)
{
	myprintf("\nCamera hardware preview setup\n\r");

	// Set preview address
	SSD_REGWL(REG_DV_VMEM_STR_ADDR1_0,addr/4);	// 0x19C
	SSD_REGWL(REG_DV_VMEM_STR_ADDR2_0,addr/4);	// 0x1A0

	// Set digital input address
	SSD_REGWL(REG_DV_DV0_START_ADDR_0,addr/4);	// 0x1B0
	SSD_REGWL(REG_DV_DV1_START_ADDR_0,addr/4);	// 0x1B4

#if BT656_input
	myprintf("Set DV input register - base on BT656\n\r");
	SSD_REGWB(REG_DV_TV_0,0x02);
	SSD_REGWB(REG_DV_TV_1,0x0d);
	SSD_REGWB(REG_DV_TV_2,0x01);
#else
	myprintf("Set DV input register - base on BT601\n\r");
	SSD_REGWB(REG_DV_TV_0,0x12);	// 0x1AC
	SSD_REGWB(REG_DV_TV_1,0x00);	// 0x1AD
	SSD_REGWB(REG_DV_TV_2,0x04);	// 0x1AE
	//SSD_REGWB(REG_DV_TV_2,0x00);
#endif

/*
	SSD_REGWW(REG_DV_NFRAME_POS_0, 	0x0002);		// 0x162
	SSD_REGWW(REG_DV_HORI_PERIOD_0, SET_CAM_WIDTH); // 0x174 -> Horizontal period
	SSD_REGWB(REG_DV_FRAME_PULSE_WIDTH, 0x08);		// 0x194 -> Vertical Width
*/

	if(on)
	{
		SSD_REGWB(REG_DV_OP_MODE,SSD_REGRB(REG_DV_OP_MODE)&~(1<<3));
		SSD_REGWB(REG_DV_OP_MODE,SSD_REGRB(REG_DV_OP_MODE)|0x12);
		myprintf("Camera Preview is ON\n\n\r");
	}
	else
	{
		SSD_REGWB(REG_DV_OP_MODE,SSD_REGRB(REG_DV_OP_MODE)&~0x12);
		myprintf("Camera Preview is OFF\n\n\r");
	}
}

/************************************************************************************************************
* void ssd192x_cam_jpeg_pause(INT32 pause)                				                                                                             *
*                                                                                                                                                                                                                      *
* Description:                                                                                                                                                                                                *
*                                                                                                                                                                                                                      *
*                                                                                                                                                                                                                      *
*                                                                                                                                                                                                                      *
*************************************************************************************************************/

void ssd192x_cam_jpeg_pause(INT32 pause)
{
	if(pause)
	{
		//JPEG and Video Still Picture Mode
		SSD_REGWB(REG_DV_OP_MODE,SSD_REGRB(REG_DV_OP_MODE)|0x09);   //REG_DV_OP_MODE = 0x160

		//Disable Preview in Camera Interface
		SSD_REGWB(REG_DV_OP_MODE,SSD_REGRB(REG_DV_OP_MODE)&~0x02);
	}
	else
	{
		SSD_REGWB(REG_DV_OP_MODE,SSD_REGRB(REG_DV_OP_MODE)&~0x01);
		SSD_REGWB(REG_DV_OP_MODE,SSD_REGRB(REG_DV_OP_MODE)|0x02); //Enable video
	}
}

/************************************************************************************************************
* void cam_reset(void)                                                                      *
*                                                                                                           *
* Description:                                                                                              *
*                                                                                                           *
* Reset camera interface by toggling between BT656 and YUV mode                                             *
*                                                                                                           *
*************************************************************************************************************/

void cam_reset(void)
{
    SSD_REGWB(REG_DV_TV_1,      0x0D);      //0x1AD
    SSD_REGWB(REG_DV_OP_MODE,   0x12);      //0x160
    SSD_REGWB(REG_DV_TV_1,      0x01);      //0x1AD
}

