#include "ssd192x.h"
#include "i2c.h"
#include "CMOS\ov7670.h"
#include "utils.h"

/*
#define YUYV					0x00
#define YVYU					0x01
#define UYVY					0x02
#define VYUY					0x03

/*********************************************
  Define the parameter of the sensor OV7670
*********************************************

#define OV7670_MODE 				OV7670_QVGA
#define OV7670_OUTPUT_FMT			YUV422
#define OV7670_OUTPUT_SEQ			UYVY


#if (OV7670_OUTPUT_FMT==YUV422)

#if (OV7670_OUTPUT_SEQ == YUYV)
#define TSLB_3				0x00
#define COM13_0				0x00
#endif

#if (OV7670_OUTPUT_SEQ == YVYU)
#define TSLB_3				0x00
#define COM13_0				0x01
#endif

#if (OV7670_OUTPUT_SEQ == UYVY)
#define TSLB_3				0x08
#define COM13_0				0x00
#endif

#if (OV7670_OUTPUT_SEQ == VYUY)
#define TSLB_3				0x08
#define COM13_0				0x01
#endif

#endif

#if (OV7670_MODE==OV7670_QVGA)
UINT32 SET_CAM_WIDTH=320;
UINT32 SET_CAM_HEIGHT=240;
UINT16 CAM_XCROP = 0;
UINT16 CAM_YCROP = 0;
UINT16 CAM_CROP_WIDTH = 320;
UINT16 CAM_CROP_HEIGHT = 240;
UINT16 CAM_VHRATIO = 0;
UINT16 CAM_VVRATIO = 0;
UINT16 CAM_JHRATIO = 0;
UINT16 CAM_JVRATIO = 0;
#endif

#if (OV7670_MODE==OV7670_VGA)
UINT32 SET_CAM_WIDTH=640;
UINT32 SET_CAM_HEIGHT=480;
UINT16 CAM_XCROP = 0;
UINT16 CAM_YCROP = 0;
UINT16 CAM_CROP_WIDTH = 640;
UINT16 CAM_CROP_HEIGHT = 480;
UINT16 CAM_VHRATIO = 1;	// preview ratio
UINT16 CAM_VVRATIO = 1;
UINT16 CAM_JHRATIO = 0;
UINT16 CAM_JVRATIO = 0;
#endif

#if (OV7670_MODE==OV7670_CIF)
UINT32 SET_CAM_WIDTH=352;
UINT32 SET_CAM_HEIGHT=288;
UINT16 CAM_XCROP = 16;
UINT16 CAM_YCROP = 24;
UINT16 CAM_CROP_WIDTH = 320;
UINT16 CAM_CROP_HEIGHT = 240;
UINT16 CAM_VHRATIO = 0;
UINT16 CAM_VVRATIO = 0;
UINT16 CAM_JHRATIO = 0;
UINT16 CAM_JVRATIO = 0;
#endif

#if (OV7670_MODE==OV7670_QCIF)
UINT32 SET_CAM_WIDTH=176;
UINT32 SET_CAM_HEIGHT=144;
UINT16 CAM_XCROP = 0;
UINT16 CAM_YCROP = 0;
UINT16 CAM_CROP_WIDTH = 176;
UINT16 CAM_CROP_HEIGHT = 144;
UINT16 CAM_VHRATIO = 0;
UINT16 CAM_VVRATIO = 0;
UINT16 CAM_JHRATIO = 0;
UINT16 CAM_JVRATIO = 0;
#endif

*/
UINT16 CAM_FRAME_RATE=30;


 struct regval_list
 {
         UINT8 reg_num;
         UINT8 value;
 };



 static struct regval_list ov7670_default_regs[] =
 {
         { REG_COM7, COM7_RESET },
 /*
  * Clock scale: 3 = 15fps
  *              2 = 20fps
  *              1 = 30fps
  */
         { REG_CLKRC, 0x1 },     			/* OV: clock scale (30 fps) */
         { REG_TSLB,  TSLB_3},    			/* UYVY */ //Reg 0x3A
         { REG_COM7,  OV7670_MODE},        	/* VGA */
         /*
          * Set the hardware window.  These values from OV don't entirely
          * make sense - hstop is less than hstart.  But they work...
          */
         { REG_HSTART, 0x13 },   { REG_HSTOP, 0x01 },
         { REG_HREF, 0xb6 },     { REG_VSTART, 0x02 },
         { REG_VSTOP, 0x7a },    { REG_VREF, 0x0a },

         { REG_COM3, 0x00},        { REG_COM14, 0 },

         /* Mystery scaling numbers */
         {0x70, 0x3a },          { 0x71, 0x35 },
         { 0x72, 0x11 },         { 0x73, 0xf0 },
         { 0xa2, 0x02 },         { REG_COM10, 0x00},

         /* Gamma curve values */
         { 0x7a, 0x20 },         { 0x7b, 0x10 },
         { 0x7c, 0x1e },         { 0x7d, 0x35 },
         { 0x7e, 0x5a },         { 0x7f, 0x69 },
         { 0x80, 0x76 },         { 0x81, 0x80 },
         { 0x82, 0x88 },         { 0x83, 0x8f },
         { 0x84, 0x96 },         { 0x85, 0xa3 },
         { 0x86, 0xaf },         { 0x87, 0xc4 },
         { 0x88, 0xd7 },         { 0x89, 0xe8 },

         /* AGC and AEC parameters.  Note we start by disabling those features,
            then turn them only after tweaking the values. */

         { REG_COM8, COM8_FASTAEC | COM8_AECSTEP | COM8_BFILT },
         { REG_GAIN, 0 },        { REG_AECH, 0 },
         { REG_COM4, 0x40 }, /* magic reserved bit */
         { REG_COM9, 0x18 }, /* 4x gain + magic rsvd bit */
         { REG_BD50MAX, 0x05 },  { REG_BD60MAX, 0x07 },
         { REG_AEW, 0x95 },      { REG_AEB, 0x33 },
         { REG_VPT, 0xe3 },      { REG_HAECC1, 0x78 },
         { REG_HAECC2, 0x68 },   { 0xa1, 0x03 }, /* magic */
         { REG_HAECC3, 0xd8 },   { REG_HAECC4, 0xd8 },
         { REG_HAECC5, 0xf0 },   { REG_HAECC6, 0x90 },
         { REG_HAECC7, 0x94 },
         { REG_COM8, COM8_FASTAEC|COM8_AECSTEP|COM8_BFILT|COM8_AGC|COM8_AEC },

         /* Almost all of these are magic "reserved" values.  */

         { REG_COM5, 0x61 },     { REG_COM6, 0x4b },
         { 0x16, 0x02 },         { REG_MVFP, 0x07 },
         { 0x21, 0x02 },         { 0x22, 0x91 },
         { 0x29, 0x07 },         { 0x33, 0x0b },
         { 0x35, 0x0b },         { 0x37, 0x1d },
         { 0x38, 0x71 },         { 0x39, 0x2a },
         { REG_COM12, 0x78 },    { 0x4d, 0x40 },
         { 0x4e, 0x20 },         { REG_GFIX, 0 },
         { 0x6b, 0x4a },         { 0x74, 0x10 },
         { 0x8d, 0x4f },         { 0x8e, 0 },
         { 0x8f, 0 },            { 0x90, 0 },
         { 0x91, 0 },            { 0x96, 0 },
         { 0x9a, 0 },            { 0xb0, 0x84 },
         { 0xb1, 0x0c },         { 0xb2, 0x0e },
         { 0xb3, 0x82 },         { 0xb8, 0x0a },

         /* More reserved magic, some of which tweaks white balance */

         { 0x43, 0x0a },         { 0x44, 0xf0 },
         { 0x45, 0x34 },         { 0x46, 0x58 },
         { 0x47, 0x28 },         { 0x48, 0x3a },
         { 0x59, 0x88 },         { 0x5a, 0x88 },
         { 0x5b, 0x44 },         { 0x5c, 0x67 },
         { 0x5d, 0x49 },         { 0x5e, 0x0e },
         { 0x6c, 0x0a },         { 0x6d, 0x55 },
         { 0x6e, 0x11 },         { 0x6f, 0x9f }, 	/* "9e for advance AWB" */
         { 0x6a, 0x40 },         { REG_BLUE, 0x40 },
         { REG_RED, 0x60 },
         { REG_COM8, COM8_FASTAEC|COM8_AECSTEP|COM8_BFILT|COM8_AGC|COM8_AEC|COM8_AWB },

         /* Matrix coefficients */
         { 0x4f, 0x80 },         { 0x50, 0x80 },
         { 0x51, 0 },            { 0x52, 0x22 },
         { 0x53, 0x5e },         { 0x54, 0x80 },
         { 0x58, 0x9e },

         { REG_COM16, COM16_AWBGAIN },
         { REG_EDGE, 0 },
         { 0x75, 0x05 },         { 0x76, 0xe1 },
         { 0x4c, 0 },            { 0x77, 0x01 },
         //{ REG_COM13, 0xc3 },    { 0x4b, 0x09 },
         { REG_COM13, COM13_0},    { 0x4b, 0x09 },	//UYVY (32Bit) (YUV422: U0(8Bit) Y0(8Bit) V0(8Bit) Y1(8Bit)
         { 0xc9, 0x60 },         { REG_COM16, 0x38 },
         { 0x56, 0x40 },

         { 0x34, 0x11 },         { REG_COM11, COM11_EXP|COM11_HZAUTO },
         { 0xa4, 0x88 },         { 0x96, 0 },
         { 0x97, 0x30 },         { 0x98, 0x20 },
         { 0x99, 0x30 },         { 0x9a, 0x84 },
         { 0x9b, 0x29 },         { 0x9c, 0x03 },
         { 0x9d, 0x4c },         { 0x9e, 0x3f },
         { 0x78, 0x04 },

         /* Extra-weird stuff.  Some sort of multiplexor register */
         { 0x79, 0x01 },         { 0xc8, 0xf0 },
         { 0x79, 0x0f },         { 0xc8, 0x00 },
         { 0x79, 0x10 },         { 0xc8, 0x7e },
         { 0x79, 0x0a },         { 0xc8, 0x80 },
         { 0x79, 0x0b },         { 0xc8, 0x01 },
         { 0x79, 0x0c },         { 0xc8, 0x0f },
         { 0x79, 0x0d },         { 0xc8, 0x20 },
         { 0x79, 0x09 },         { 0xc8, 0x80 },
         { 0x79, 0x02 },         { 0xc8, 0xc0 },
         { 0x79, 0x03 },         { 0xc8, 0x40 },
         { 0x79, 0x05 },         { 0xc8, 0x30 },
         { 0x79, 0x26 },

         { 0xff, 0xff }, /* END MARKER */
 };


 /*
  * Here we'll try to encapsulate the changes for just the output
  * video format.
  *
  * RGB656 and YUV422 come from OV; RGB444 is homebrewed.
  *
  * IMPORTANT RULE: the first entry must be for COM7, see ov7670_s_fmt for why.
  */


 static struct regval_list ov7670_fmt_yuv422[] =
 {

         { REG_COM7, 0x0},  		/* Selects YUV mode */
         { REG_RGB444, 0 },      	/* No RGB444 please */
         { REG_COM1, 0 },
         { REG_COM15, COM15_R00FF },
         { REG_COM9, 0x18 },     	/* 4x gain ceiling; 0x8 is reserved bit */
         { 0x4f, 0x80 },         	/* "matrix coefficient 1" */
         { 0x50, 0x80 },         	/* "matrix coefficient 2" */
         { 0x51, 0    },         	/* vb */
         { 0x52, 0x22 },         	/* "matrix coefficient 4" */
         { 0x53, 0x5e },         	/* "matrix coefficient 5" */
         { 0x54, 0x80 },         	/* "matrix coefficient 6" */
         //{ REG_COM13, COM13_GAMMA|COM13_UVSAT },
         { 0xff, 0xff },
 };



void cam_hw_init(void)
{

	I2C i2c;
	UINT16 i = 0;
	UINT16 error = 0;

	//Reset the DV enable module

	SSD_REGWB(REG_DV_ENB,0x01);	// ENB_DV = 1= RESET_DV = 1
	SSD_REGWB(REG_DV_ENB,0x00);	// ENB_DV = 0= RESET_DV = 0
	mdelay(100);
	SSD_REGWB(REG_DV_ENB,0x01);	// ENB_DV = 1= RESET_DV = 1

	// Set the CLK

	SSD_REGWB(REG_VCLK_CONFIG_0,0x03);  // Reg 0x06 Cam running rate
	SSD_REGWB(REG_VCLK_CONFIG_1,0x07);  // Reg 0x07 - Enable the VCLK

	// enable I2C

	i2c.deviceID=I2C_DEVICE_ID;
	i2c.bitRate=1;
  	ssd192x_i2c_init(&i2c);

  	mdelay(100);
  	myprintf("Configure OV7670 via I2C\n\r");
  	myprintf("VER: %x",ssd192x_i2c_readb_without_ACK(&i2c,REG_MIDH));
  	mdelay(100);
  	myprintf("  PID: %x\n\r",ssd192x_i2c_readb_without_ACK(&i2c,REG_MIDL));

  	i = 0;

  	while (ov7670_default_regs[i].reg_num != 0xff || ov7670_default_regs[i].value != 0xff)
  	{
      		if(ssd192x_i2c_writeb(&i2c,ov7670_default_regs[i].reg_num, ov7670_default_regs[i].value))
				error++;
      		i++;
  	}

	if(error > 0)
		myprintf("Program OV7670 register error %d\r\n",error);

	error = 0;
	i = 0;

 	myprintf("Config YUV mode \r\n");

  	while (ov7670_fmt_yuv422[i].reg_num != 0xff || ov7670_fmt_yuv422[i].value != 0xff)
  	{
      		if(ssd192x_i2c_writeb(&i2c,ov7670_fmt_yuv422[i].reg_num, ov7670_fmt_yuv422[i].value))
				error++;
      		i++;
  	}

	if(error > 0)
		myprintf("Program OV7670 register error %d\r\n",error);

	//myprintf("TSLB = %x",ssd192x_i2c_readb_without_ACK(&i2c,REG_TSLB));
	//myprintf("  COM13 = %x\r\n\n",ssd192x_i2c_readb_without_ACK(&i2c,REG_COM13));
}

/*
void cam_hw_still(void)
{

#if BT656_input
	myprintf("Set DV input register - base on BT656\n\r");
	SSD_REGWB(REG_DV_TV_0,0x02);
	SSD_REGWB(REG_DV_TV_1,0x0d);
	SSD_REGWB(REG_DV_TV_2,0x01);
#else
	myprintf("Set DV input register - base on BT601\n\r");
	SSD_REGWB(REG_DV_TV_0,0x12);	// 0x1AC
	SSD_REGWB(REG_DV_TV_1,0x00);	// 0x1AD
	SSD_REGWB(REG_DV_TV_2,0x04);	// 0x1AE
	//SSD_REGWB(REG_DV_TV_2,0x00);
#endif

	SSD_REGWW(REG_DV_NFRAME_POS_0, 	0x0002);		// 0x162
	SSD_REGWW(REG_DV_HORI_PERIOD_0, SET_CAM_WIDTH); // 0x174 -> Horizontal period
	SSD_REGWB(REG_DV_FRAME_PULSE_WIDTH, 0x08);		// 0x194 -> Vertical Width
}
*/
