#include "ssd192x.h"
#include "camera.h"
#include "UART0.h"
#include "utils.h"
#include "type.h"
#include "lcd.h"
#include "io.h"

#include "include/test_routine.h"


extern UINT32 SET_CAM_WIDTH;
extern UINT32 SET_CAM_HEIGHT;
extern UINT16 CAM_XCROP;
extern UINT16 CAM_YCROP;
extern UINT16 CAM_CROP_WIDTH;
extern UINT16 CAM_CROP_HEIGHT;
extern UINT16 CAM_VHRATIO;	// preview ratio
extern UINT16 CAM_VVRATIO;
extern UINT16 CAM_JHRATIO;
extern UINT16 CAM_JVRATIO;


/************************************************************************************************************
* void SSD1928ADS_Camera_Test(void)                                                                                             *
*                                                                                                           *
* Description:                                                                                              *
*                                                                                                           *
*                                                                                                           *
*                                                                                                           *
*************************************************************************************************************/

void SSD1921DVK_Camera_Test(void)
{
	INT32 cam_memory_offset = 0;
	INT32 vhdivider=1;
	INT32 vvdivider=1;
	INT32 vh;
	INT32 vv;
	INT32 floatWidth;
	INT32 floatHeight;

	// the CAM_VxRATIO define how much scaling to be done, which is set at registers 171 & 172

	//Init_LCD();	// clear the screen
	ssd192x_lcd_blank_enable(1);

	for(vh=0;vh<CAM_VHRATIO;vh++)
	{
		vhdivider*=2;
	}

	for(vv=0;vv<CAM_VVRATIO;vv++)
	{
		vvdivider*=2;
	}

	/* Base on the divider ratio, decide the display size of the floating window */

	floatWidth = CAM_CROP_WIDTH / vhdivider;
	floatHeight = CAM_CROP_HEIGHT / vvdivider;

	myprintf("\nDevice input width 	= %d\r\n",SET_CAM_WIDTH);
	myprintf("Device input height 	= %d\r\n",SET_CAM_HEIGHT);
	myprintf("Display window width  = %d\r\n",floatWidth);
	myprintf("Display window height  = %d\r\n",floatHeight);

	/*Capture whole 640x480 picture and scale down to 320x240 (Using 320x240x2 memory buffer)
     * Display this picture using Floating window
     */

    ssd192x_cam_init(CAM_XCROP, CAM_YCROP, CAM_CROP_WIDTH, CAM_CROP_HEIGHT, CAM_VHRATIO, CAM_VVRATIO, CAM_JHRATIO, CAM_JVRATIO);

    //main window is overlay with float window, icons can be put at upper part and lower part
    //main window buffer is located just after the camera preview buffer

    ssd192x_lcd_init(0x3FFFF,0,0,0);	// will set the background of the LCD all green for YUV
    ssd192x_lcd_blank_enable(0);

    //Initialize float window, overlay on main window
    ssd192x_floatwin_init(cam_memory_offset,floatWidth,0,0,(LCD_WIDTH-floatWidth)/2,(LCD_HEIGHT-floatHeight)/2,floatWidth,floatHeight);

    //start the preview
    ssd192x_cam_preview(cam_memory_offset, 1); // code in camera.c

}


